#include "WhitelistFilter.h"

#include <SimoxUtility/algorithm/string.h>

namespace armarx::aron::data::filter
{
    WhitelistFilter::WhitelistFilter(const std::vector<std::string>& whitelist,
                                     const std::map<std::string, Normalization>& normalization) :
        whitelist(whitelist), normalization(normalization)
    {
    }

    bool
    WhitelistFilter::pathInWhitelist(const aron::Path& p) const
    {
        for (const auto& w : whitelist)
        {
            if (simox::alg::starts_with(p.toString(), w))
            {
                return true;
            }
        }
        return false;
    }

    void
    WhitelistFilter::visitAronVariant(const data::ListPtr& aron)
    {
        auto l = aron::make_list(aron->getPath());
        for (const auto& el : aron->getElements())
        {
            WhitelistFilter v(whitelist, normalization);
            aron::data::visit(v, el);

            if (v.data)
            {
                l->addElement(v.data);
            }
        }

        if (l->getElements().size() || pathInWhitelist(aron->getPath()))
        {
            data = l;
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::DictPtr& aron)
    {
        auto l = aron::make_dict(aron->getPath());
        for (const auto& [k, el] : aron->getElements())
        {
            WhitelistFilter v(whitelist, normalization);
            aron::data::visit(v, el);

            if (v.data)
            {
                l->addElement(k, v.data);
            }
        }

        if (l->getElements().size() || pathInWhitelist(aron->getPath()))
        {
            data = l;
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::NDArrayPtr& aron)
    {
        if (pathInWhitelist(aron->getPath()))
        {
            data = aron->clone();
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::IntPtr& aron)
    {
        if (pathInWhitelist(aron->getPath()))
        {
            auto e = aron->clone();
            int v = checkForNormalization(aron->getPath(), e->getValue());
            e->setValue(v);
            data = e;
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::LongPtr& aron)
    {
        if (pathInWhitelist(aron->getPath()))
        {
            auto e = aron->clone();
            long v = checkForNormalization(aron->getPath(), e->getValue());
            e->setValue(v);
            data = e;
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::FloatPtr& aron)
    {
        if (pathInWhitelist(aron->getPath()))
        {
            auto e = aron->clone();
            float v = checkForNormalization(aron->getPath(), e->getValue());
            e->setValue(v);
            data = e;
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::DoublePtr& aron)
    {
        if (pathInWhitelist(aron->getPath()))
        {
            auto e = aron->clone();
            double v = checkForNormalization(aron->getPath(), e->getValue());
            e->setValue(v);
            data = e;
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::BoolPtr& aron)
    {
        if (pathInWhitelist(aron->getPath()))
        {
            auto e = aron->clone();
            bool v = checkForNormalization(aron->getPath(), e->getValue());
            e->setValue(v);
            data = e;
        }
    }

    void
    WhitelistFilter::visitAronVariant(const data::StringPtr& aron)
    {
        if (pathInWhitelist(aron->getPath()))
        {
            data = aron->clone();
        }
    }
} // namespace armarx::aron::data::filter
